@@ -439,6 +439,9 @@ In addition, the following extra syntax elements are enabled:
 [GitHub-Flavored Markdown]: https://github.github.com/gfm/
 [PHP-Markdown Extra]: https://michelf.ca/projects/php-markdown/extra/#table
 
+It is possible (but not recommended) to use another Markdown library with pdoc.
+See [#401](https://github.com/mitmproxy/pdoc/issues/401#issuecomment-1148661829) for details.
+
 # Using pdoc as a library
 
 pdoc provides the high-level `pdoc.pdoc()` interface explained below. This makes it possible to do custom adjustments
@@ -450,7 +453,7 @@ You can find an example in [`examples/library-usage`](https://github.com/mitmpro
 from __future__ import annotations
 
 __docformat__ = "markdown"  # explicitly disable rST processing in the examples above.
-__version__ = "12.2.2"  # this is read from setup.py
+__version__ = "12.3.1"  # this is read from setup.py
 
 from pathlib import Path
 from typing import overload
