@@ -12,7 +12,10 @@ accompanied by matching snapshot tests in `test/testdata/`.
 """
 from __future__ import annotations
 
+import base64
 import inspect
+import mimetypes
+import os
 import re
 import warnings
 from pathlib import Path
@@ -38,9 +41,32 @@ def convert(docstring: str, docformat: str, source_file: Path | None) -> str:
     if "numpy" in docformat:
         docstring = numpy(docstring)
 
+    if source_file is not None and os.environ.get("PDOC_EMBED_IMAGES") != "0":
+        docstring = embed_images(docstring, source_file)
+
     return docstring
 
 
+def embed_images(docstring: str, source_file: Path) -> str:
+    def embed_local_image(m: re.Match) -> str:
+        image_path = source_file.parent / m["href"]
+        try:
+            image_data = image_path.read_bytes()
+            image_mime = mimetypes.guess_type(image_path)[0]
+        except Exception:
+            return m[0]
+        else:
+            data = base64.b64encode(image_data).decode()
+            return f"![{m['alt']}](data:{image_mime};base64,{data})"
+
+    return re.sub(
+        r"!\[\s*(?P<alt>.*?)\s*]\(\s*(?P<href>.+?)\s*\)",
+        embed_local_image,
+        docstring,
+    )
+    # TODO: Could probably do more here, e.g. support rST or raw HTML replacements.
+
+
 def google(docstring: str) -> str:
     """Convert Google-style docstring sections into Markdown."""
     return re.sub(
@@ -59,11 +85,27 @@ def google(docstring: str) -> str:
 
 
 GOOGLE_LIST_SECTIONS = ["Args", "Raises", "Attributes"]
+"""Section headers listed in the official Google docstring style guide."""
+
+GOOGLE_LIST_SECTION_ALIASES = {
+    "Parameters": "Args",
+    "Params": "Args",
+    "Arguments": "Args",
+}
+"""
+Alternative section headers that are not listed in the official Google
+docstring style guide but that we recognize as sections containing lists
+nevertheless.
+"""
 
 
 def _google_section(m: re.Match[str]) -> str:
     name = m.group("name")
-    contents = dedent(m.group("contents"))
+    contents = dedent(m.group("contents")).lstrip()
+
+    if name in GOOGLE_LIST_SECTION_ALIASES:
+        name = GOOGLE_LIST_SECTION_ALIASES[name]
+
     if name in GOOGLE_LIST_SECTIONS:
         items = _indented_list(contents)
         contents = ""
@@ -104,9 +146,8 @@ def _indented_list(contents: str) -> list[str]:
     ]
     """
     # we expect this to be through cleandoc() already.
-    assert not contents.startswith(" ")
-    assert not contents.startswith("\n")
-    assert "\t" not in contents
+    assert not contents.startswith(" "), contents
+    assert not contents.startswith("\n"), contents
 
     ret: list[str] = []
     for line in contents.splitlines(keepends=True):
@@ -212,9 +253,18 @@ def rst(contents: str, source_file: Path | None) -> str:
     contents = _rst_admonitions(contents, source_file)
     contents = _rst_links(contents)
 
+    def replace_reference(m):
+        _, kind, name = m.groups()
+        if kind in ("meth", "func"):
+            return f"`{name}()`"
+        else:
+            return f"`{name}`"
+
     # Code References: :obj:`foo` -> `foo`
     contents = re.sub(
-        r"(:py)?:(mod|func|data|const|class|meth|attr|exc|obj):", "", contents
+        r"(:py)?:(mod|func|data|const|class|meth|attr|exc|obj):`([^`]+)`",
+        replace_reference,
+        contents,
     )
 
     # Math: :math:`foo` -> \\( foo \\)
