@@ -1,16 +1,20 @@
 from __future__ import annotations
 
+import html
 import inspect
 import os
 import re
 import warnings
-from collections.abc import Collection, Iterable, Mapping
+from collections.abc import Collection
+from collections.abc import Iterable
+from collections.abc import Mapping
 from contextlib import contextmanager
 from unittest.mock import patch
 
 import pygments.formatters
 import pygments.lexers
-from jinja2 import ext, nodes
+from jinja2 import ext
+from jinja2 import nodes
 
 try:
     # Jinja2 >= 3.0
@@ -50,12 +54,14 @@ The pygments formatter used for pdoc.render_helpers.format_signature.
 Overwrite this to configure pygments highlighting of signatures.
 """
 
+# Keep in sync with the documentation in pdoc/__init__.py!
 markdown_extensions = {
     "code-friendly": None,
     "cuddled-lists": None,
     "fenced-code-blocks": {"cssclass": formatter.cssclass},
     "footnotes": None,
     "header-ids": None,
+    "link-patterns": None,
     "markdown-in-html": None,
     "pyshell": None,
     "strike": None,
@@ -67,6 +73,27 @@ markdown_extensions = {
 The default extensions loaded for `markdown2`.
 Overwrite this to configure Markdown rendering.
 """
+markdown_link_patterns = [
+    (
+        re.compile(
+            r"""
+            \b
+            (
+                (?:https?://|(?<!//)www\.)    # prefix - https:// or www.
+                \w[\w_\-]*(?:\.\w[\w_\-]*)*   # host
+                [^<>\s"']*                    # rest of url
+                (?<![?!.,:*_~);])             # exclude trailing punctuation
+                (?=[?!.,:*_~);]?(?:[<\s]|$))  # make sure that we're not followed by " or ', i.e. we're outside of href="...".
+            )
+        """,
+            re.X,
+        ),
+        r"\1",
+    )
+]
+"""
+Link pattern used for markdown2's [`link-patterns` extra](https://github.com/trentm/python-markdown2/wiki/link-patterns).
+"""
 
 
 @cache
@@ -102,7 +129,7 @@ def format_signature(sig: inspect.Signature, colon: bool) -> str:
         if '<span class="err">' not in pretty:
             return pretty
         else:
-            return code
+            return html.escape(code)
 
     # Next, individually highlight each parameter using pygments and wrap it in a span.param.
     # This later allows us to properly control line breaks.
@@ -120,13 +147,15 @@ def format_signature(sig: inspect.Signature, colon: bool) -> str:
         pretty_result[-1] = pretty_result[-1].rpartition(",")[0] + "</span>"
 
     # Add return annotation.
-    rendered = "(%s)" % "".join(pretty_result)
+    anno = ")"
     if return_annot:
-        anno = _try_highlight(return_annot)
-        rendered = (
-            rendered[:-1]
-            + f'<span class="return-annotation">) -> {anno}{":" if colon else ""}</span>'
-        )
+        anno += f" -> {_try_highlight(return_annot)}"
+    if colon:
+        anno += ":"
+    if return_annot or colon:
+        anno = f'<span class="return-annotation">{anno}</span>'
+
+    rendered = "(" + "".join(pretty_result) + anno
 
     if multiline:
         rendered = f'<span class="signature pdoc-code multiline">{rendered}</span>'
@@ -144,7 +173,11 @@ def to_html(docstring: str) -> str:
     # careful: markdown2 returns a subclass of str with an extra
     # .toc_html attribute. don't further process the result,
     # otherwise this attribute will be lost.
-    return pdoc.markdown2.markdown(docstring, extras=markdown_extensions)  # type: ignore
+    return pdoc.markdown2.markdown(  # type: ignore
+        docstring,
+        extras=markdown_extensions,
+        link_patterns=markdown_link_patterns,
+    )
 
 
 @pass_context
@@ -248,17 +281,17 @@ def linkify(context: Context, code: str, namespace: str = "") -> str:
 
     def linkify_repl(m: re.Match):
         text = m.group(0)
-        identifier = removesuffix(text, "()")
-        identifier = identifier.replace(
+        plain_text = text.replace(
             '</span><span class="o">.</span><span class="n">', "."
         )
+        identifier = removesuffix(plain_text, "()")
 
         # Check if this is a local reference within this module?
         mod: pdoc.doc.Module = context["module"]
         for qualname in qualname_candidates(identifier, namespace):
             doc = mod.get(qualname)
             if doc and context["is_public"](doc).strip():
-                return f'<a href="#{qualname}">{text}</a>'
+                return f'<a href="#{qualname}">{plain_text}</a>'
 
         module = ""
         qualname = ""
@@ -273,11 +306,11 @@ def linkify(context: Context, code: str, namespace: str = "") -> str:
                     and doc.taken_from == (module, qualname)
                     and context["is_public"](doc).strip()
                 ):
-                    if text.endswith("()"):
-                        text = f"{doc.fullname}()"
+                    if plain_text.endswith("()"):
+                        plain_text = f"{doc.fullname}()"
                     else:
-                        text = doc.fullname
-                    return f'<a href="#{qualname}">{text}</a>'
+                        plain_text = doc.fullname
+                    return f'<a href="#{qualname}">{plain_text}</a>'
         except ValueError:
             # possible_sources did not find a parent module.
             return text
@@ -293,7 +326,7 @@ def linkify(context: Context, code: str, namespace: str = "") -> str:
             if target_exists_and_public:
                 if qualname:
                     qualname = f"#{qualname}"
-                return f'<a href="{relative_link(context["module"].modulename, module)}{qualname}">{text}</a>'
+                return f'<a href="{relative_link(context["module"].modulename, module)}{qualname}">{plain_text}</a>'
             else:
                 return text
 
@@ -336,8 +369,18 @@ def link(context: Context, spec: tuple[str, str], text: str | None = None) -> st
     modulename, qualname = spec
 
     # Check if the object we are interested is also imported and re-exposed in the current namespace.
-    doc = mod.get(qualname)
-    if doc and doc.taken_from == spec and context["is_public"](doc).strip():
+    # https://github.com/mitmproxy/pdoc/issues/490: We need to do this for every level, not just the tail.
+    doc: pdoc.doc.Doc | None = mod
+    for part in qualname.split("."):
+        doc = doc.get(part) if isinstance(doc, pdoc.doc.Namespace) else None
+        if not (
+            doc
+            and doc.taken_from[0] == modulename
+            and context["is_public"](doc).strip()
+        ):
+            break
+    else:
+        # everything down to the tail is imported and re-exposed.
         if text:
             text = text.replace(f"{modulename}.", f"{mod.modulename}.")
         modulename = mod.modulename
